# -*- coding: utf-8 -*-
from setuphelpers import *


def update_package():
    # Declaring local variables
    package_updated = False
    proxies = get_proxies()
    if not proxies:
        proxies = get_proxies_from_wapt_console()
    url = "https://download.virtualbox.org/virtualbox/"
    vbox_fixed_version = "6"
    download_dict = {
        "windows-x64": "-Win.exe",
        "darwin-x64": "-OSX.dmg",
        "debian-stretch-x64": "stretch",
        "debian-buster-x64": "buster",
        "debian-bullseye-x64": "bullseye",
        "centos8,centos-8,almalinux-x64": "el8",
        "centos7-x64": "el7",
    }

    # Getting latest "vbox_fixed_version"'s version
    print("URL used is: %s" % url)
    url_dict = bs_index_of_to_dict(url, proxies=proxies)
    version = Version("0")
    for key in url_dict.keys():
        if key.startswith(vbox_fixed_version) and Version(key.strip("/")) > Version(version):
            version = key.strip("/")
    version_url = url_dict[version + "/"]

    binaries_dict = bs_index_of_to_dict(version_url, proxies=proxies)
    for elem in binaries_dict.values():
        # Getting Vbox binary download link
        if download_dict[control.target_os + "-" + ensure_list(control.architecture)[0]] in elem:
            download_url = elem
            latest_bin = download_url.split("/")[-1]
        # Getting Vbox extensions pack binary download link
        if ".vbox-extpack" in elem:
            download_ext_url = elem
            latest_ext_bin = download_ext_url.split("/")[-1]

    # Downloading latest binaries
    print("Latest %s version is: %s" % (control.name, version))
    print("Download URL is: %s" % download_url)
    if not isfile(latest_bin):
        print("Downloading: %s" % latest_bin)
        wget(download_url, latest_bin, proxies=proxies)
    else:
        print("Binary is present: %s" % latest_bin)

    # Downloading extension pack
    print("Download Extension Pack URL is: %s" % download_ext_url)
    if not isfile(latest_ext_bin):
        print("Downloading: %s" % latest_ext_bin)
        wget(download_ext_url, latest_ext_bin, proxies=proxies)
    else:
        print("Extension pack binary is present: %s" % latest_ext_bin)

    # Deleting outdated binaries
    remove_outdated_binaries(version, list_extensions=[download_dict[control.target_os + "-" + ensure_list(control.architecture)[0]], "vbox-extpack"])

    # Checking version from file
    if get_os_version() == "Windows":
        version_from_file = get_version_from_binary(latest_bin)
        if Version(version_from_file, 4) != Version(version, 4) and version_from_file != "":
            print("ERROR: Binary file version do NOT corresponds to online version")
        else:
            print("INFO: Binary file version corresponds to online version")

    # Changing version of the package
    if Version(version, 4) > Version(control.get_software_version(), 4):
        print("Software version updated (from: %s to: %s)" % (control.get_software_version(), Version(version)))
        package_updated = True
    else:
        print("Software version up-to-date (%s)" % Version(version))
    control.set_software_version(version)
    control.save_control_to_wapt()

    # Validating or not update-package-sources
    return package_updated


def bs_index_of_to_dict(url, **kwargs):
    r"""Parse html "Index of" web page with BeautifulSoup and get a dict of the result

    Args:
        url (str): url of the web page to parse

    Return:
        result: a dict of the index URLs

    """
    result = {}
    if not url.endswith("/"):
        url = url + "/"

    for elem in bs_find_all(url, "a"):
        result[elem.text] = url + elem.get("href", "")

    return result
